class Mode:
    NORMAL = 'normal'
    BOLD = 'bold'
    UNDERLINE = 'underline'
    BLINK = 'blink'
    INVERT = 'invert'
    HIDE = 'hide'

    def __repr__(self):
        return '123'

    def __str__(self):
        return '123'


print(Mode)
